<?php

/*
  --------------------------------------------------------------------------
  GPS Ellipsoid 3D XYZ Calculator

  This program computes the rectangular 3D XYZ-coordinates corresponding
  to any given latitude, longitude coordinates and elevation on the
  GPS/GRS-80/WGS-84 ellipsoid model.

  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International license.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------
*/


   $_BaseScriptName_ = BaseName($_SERVER['SCRIPT_FILENAME']);
   $_ProgramVersion_ = 'v1.10';
   $at = "&#97;&#116;";
   $_RevisionDate_ = gmdate("Y F d  l $at h:i:s A", FileMTime($_BaseScriptName_)).' GMT';

   $cYear = date('Y');

// GET NAME OF PROGRAM SCRIPT TO RUN
// WHEN THE SUBMIT BUTTON IS CLICKED.
// IN THIS CASE, IT MEANS THIS SCRIPT.
   $_RUN_ = $_SERVER['SCRIPT_NAME'];

   $ButtonClicked  = '';
   $ButtonClicked .= @$_POST['ComputeButton'];

   $ButtonClicked  = trim($ButtonClicked);



// SET INITIAL DEFAULT STARTUP MODEL TO GRS80/WGS84
   if ($ButtonClicked == '') {$ButtonClicked = "C O M P U T E";}

   $LatLon    = @trim($_POST['LatLon']);
   $elevation = @strtolower(trim($_POST['Elevation']));


// SET INITIAL DEFAULT STARTUP COORDINATES
   if ($LatLon == '' and $elevation == '')
      {
        $LatLon    = '+42.6764593056 ,  -76.8230111111'; // Set home defaults
        $Elevation = '+261.823 m'; // Set home default

   // DEFAULT OVERRIDES FOR TESTING AND CALIBRATION.
        $LatLon    = '+53.6119903611 ,  -1.6644422222'; // Control test defaults
        $Elevation = '+299.800 m'; // Control Test dfault

        $LatLon = Parse_Lat_Lon_String($LatLon);

        $elevation = floatval($Elevation);
      }

   $LatLon = str_replace("°",      " ", $LatLon);
   $LatLon = str_replace("&#176;", " ", $LatLon);
   $LatLon = str_replace("&deg;",  " ", $LatLon);

   $LatLon = Parse_Lat_Lon_String($LatLon);

   list($LatDeg, $LonDeg) = preg_split("[,]", $LatLon);
   $LonSign  = ($LonDeg < 0)? '-' : '+';
   $LonSignV = ($LonDeg < 0)?  -1  : 1;

   $LatDeg = DMS_to_Deg($LatDeg);
   $LonDeg = DMS_to_Deg($LonDeg);
   $LatDMS = Deg_to_DMS($LatDeg, 4, '+', 'd');
   $LatDeg = sprintf("%+3.10f", $LatDeg);

   $LonDMS = Deg_to_DMS($LonDeg, 4, '+', 'd');
   $LonDMS = Deg_to_DMS(floatval($LonSignV*$LonDeg), 4, '+', 'd');
   $LonDeg = sprintf("%+3.10f", floatval($LonSignV*$LonDeg));

   $LatLon = "$LatDeg ,  $LonDeg";

// CHECK ELEVATION ARGUMENT.  IF FEET (ft) INDICATED,
// THEN CONVERT VALUE INTO EQUIVALENT METERS.
   $w = (strpos($elevation, 'f') === FALSE)? 1 : 0.3048;
   $elev = floatval($elevation) * $w;

// SET N/S/E/W indicators for lat/lon values
   $SN = ($LatDeg < 0)? 'S':'N';  if ($LatDeg == 0){$SN = ' ';}
   $WE = ($LonDeg < 0)? 'W':'E';  if ($LonDeg == 0){$WE = ' ';}


   $StatsStr = $description = '';

// CALL FUNCTION TO COMPUTE 3D XYZ COORDINATES.
   if ($ButtonClicked == "C O M P U T E")
      {$StatsStr = XYZ_For_GRS80_WGS84 ($LatDeg, $LonDeg, $elev); $description = "Used by the ITRS - International Terrestrial Reference System and\nthe GPS - Global Positioning System";}

       $elevation = sprintf("%+1.4f", $elev).' m';


// PARSE RETURNED ELLIPSOID STATS STRING

   list
   (
    $x, $y, $z,
    $a, $b, $H,
    $f, $F,
    $e, $e2
   ) = preg_split("[ ]", strip_wspace($StatsStr));

   $xkm = sprintf("%+1.6f", $x/1000);
   $ykm = sprintf("%+1.6f", $y/1000);
   $zkm = sprintf("%+1.6f", $z/1000);

   $Hft = $H / 0.3048;  $w = ($H < 0)? '-':'+';
   $hft = floor(abs($Hft));
   $hin = sprintf("%1.1f", 12*(abs($Hft) - $hft));
   if ($hin == 12){$hft++; $hin = '0.0';}
   $Hftin = "$w$hft ft and $hin in";

   $w   = 1609.344; // EXACT meters per mile
   $ami = $a / $w;
   $bmi = $b / $w;
   $ami = sprintf("%1.6f", $ami);
   $bmi = sprintf("%1.6f", $bmi);

   $a   = sprintf("%1.4f",  $a);
   $b   = sprintf("%1.4f",  $b);
   $e2  = sprintf("%1.12f", $e2);
   $OneMinuse = sprintf("%1.12f",  1 - $e);
   $OneMinuse2 = sprintf("%1.12f", 1 - $e2);
   $e   = sprintf("%1.12f", $e);

   $f   = sprintf("%1.12f", $f);
   $H   = sprintf("%+1.4f", $H);
   $Hft = sprintf("%+1.2f", $Hft);

   $xmi = $x / $w;
   $ymi = $y / $w;
   $zmi = $z / $w;

   $x   = sprintf("% +12.3f", $x);
   $y   = sprintf("% +12.3f", $y);
   $z   = sprintf("% +12.3f", $z);
   $xmi = sprintf("% +12.6f", $xmi);
   $ymi = sprintf("% +12.6f", $ymi);
   $zmi = sprintf("% +12.6f", $zmi);

   $xkm = sprintf("% +12.6f", $xkm);
   $ykm = sprintf("% +12.6f", $ykm);
   $zkm = sprintf("% +12.6f", $zkm);

   $F = sprintf("%1.12f", $F);







   $OutputTextArea =
"GPS/GRS-80/WGS-84 Ellipsoid Model 3D XYZ−Coordinates

$description

------------------------------------------------------------------
Location Geographic Coordinates and Elevation

Latitude   =  $LatDMS $SN  =  $LatDeg&deg;
Longitude  =  $LonDMS $WE  =  $LonDeg&deg;

Elevation  =  $H m  =  $Hft ft =  $Hftin

--------------------------------------------
Corresponding Rectangular 3D XYZ-Coordinates

x  = $x m  =  $xmi mi  =  $xkm km
y  = $y m  =  $ymi mi  =  $ykm km
z  = $z m  =  $zmi mi  =  $zkm km

------------------------------------------------------------------
GPS/GRS-80/WGS-84 Ellipsoid Parameters

a = Equatorial radius  =  $a m  =  $ami mi
b = Polar radius       =  $b m  =  $bmi mi
f = Flattening         =  $f  =  1/$F
e = Eccentricity       =  $e
                    e&#178; =  $e2
";



// _______________________________________________________
// DETERMINE HOW MANY ROWS TO USE FOR THE OUTPUT TEXTAREA.
   $TextAreaRows = 2 + substr_count($OutputTextArea, "\n");




print <<< _HTML


<!DOCTYPE HTML>

<html>

<head>

<meta http-equiv="content-type" content="text/html; charset=UTF-8">

<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<title>PHP Ellipsoid 3D XYZ-Coordinates calculator - NeoProgrammics.com</title>


<style type='text/css'>

BODY {color:white; background-color:black;  font-family:Verdana; font-size:14px}

TABLE {font-family:Verdana; font-size:14px;}

TD {color:black; background-color:white;  line-height:125%; font-size:14px; padding:6px; text-align:center;}

LI {text-align:justify;}

DIV, P {color:black; background-color:white; font-family:Verdana; text-align:left; padding:6px; font-size:14px;}

PRE {color:black; background-color:white; font-family:monospace; font-size:16px; font-weight:bold; text-align:left; border:1px solid black; border-radius:8px; box-shadow:4px 4px 3px #999999; padding:4px;}

CODE {font-family:monospace; font-size:16px; font-weight:bold;}

TEXTAREA {color:black; background-color:white; font-family:monospace; font-size:18px; font-weight:bold; box-shadow:4px 4px 3px #999999; border-radius: 8px; border:1px solid black; padding:4px; white-space:pre;}

IMG {border:1px solid silver; box-shadow:2px 2px 3px #999999; border-radius:8px;}

INPUT[type='text']::-ms-clear {width:0; height:0;}
INPUT[type='text']{color:black; background-color:white; font-family:monospace; font-size:18px; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #999999; border:2px solid black;}
INPUT[type='text']:focus {color:black; background-color:white; font-family:monospace; font-size:18px; font-weight:bold; text-align:center; box-shadow:2px 2px 3px #666666; border:2px solid red;}

INPUT[type='submit']{font-weight:bold;}
INPUT[type='submit']:hover {box-shadow:2px 2px 5px #222222; font-weight:bold;}

INPUT[type='checkbox']:checked {box-shadow:2px 2px 3px #222222;}

INPUT[type='radio']:checked {box-shadow:2px 2px 3px #222222;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */

</style>


</head>

<body>

<form name="EllipsoidForm" method="post" action="$_RUN_">

<table width="715" border="0" cellspacing="1" cellpadding="3">

<tr><td colspan="2" style='color:white; background-color:#0000BB; border:2px solid white; border-radius:8px 8px 0px 0px;'><b style='font-size:18px; line-height:150%;'>GPS Ellipsoid 3D XYZ&minus;Coordinates Calculator</b><br><b>For Any Point Relative to the Surface of the GPS/GRS-80/WGS-84 Ellipsoid</b></td></tr>

<!--
<tr><td colspan="2">&nbsp;</td></tr>
-->

<tr>
<td width="193"><b style='font-size:16px;'>Latitude</b>&nbsp;&nbsp;(&minus;Neg = South)&nbsp;&nbsp; <span style='font-size:18px;'>,</span>&nbsp; <b style='font-size:16px;'>Longitude</b>&nbsp;&nbsp;(&minus;Neg = West)<br>
±Deg.ddddd
&nbsp; <b>or</b> &nbsp; &plusmn;Deg min sec
<input name="LatLon" type="text" value="$LatLon" size="39" maxlength="38"></td>
<td width="130"><b style='font-size:14px;'>Sea-Level Elevation</b>&nbsp;<br>in meters or feet (<b>m</b>, &nbsp; <b>ft</b>)
<input name="Elevation" type="text" value="$elevation" size="17" maxlength="16"></td>
</tr>



<tr>
<td colspan='2' style='line-height:300%; background-color:#383032;'>
<input name='ComputeButton' type='submit' value='C O M P U T E'>
</td>
</tr>

<tr><td colspan="2">&nbsp;</td></tr>

<tr><td colspan="2" style='background-color:GreenYellow; padding:16px;'>
<textarea name="OutputTextArea" cols="70" rows="$TextAreaRows" readonly='readonly'>$OutputTextArea</textarea>
</td></tr>

<tr><td colspan="2" style='text-align:center;'>

<b>Basic Ellipsoid Equations Applied</b>
<br><br>
<img src='images/basic_ellipsoid_formulas.png' alt=''>

<br><br>

</td></tr>

<tr><td colspan='2' style='color:silver; background-color:black;'>Jay Tanner - $cYear</td></tr>



</table>
</form>

<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>

</body>
</html>


_HTML;



/*

   This is a local utility function to parse the
   (LatLonString) argument string before using it.
*/

   function Parse_Lat_Lon_String($LatLonString)
{

// READ INPUT ARGUMENT STRING AND STRIP ALL WHITE SPACE.
   $LatLonStr = strip_wspace($LatLonString);

   $LatLonStr = str_replace(',', ' , ', $LatLonStr);

// IF EMPTY INPUT, THEN SET DEFAULT COORDINATES TO (0 , 0).
   if ($LatLonStr == '')
      {return '0.0000000000 , 0.0000000000';}


// COUNT ALL SPACES.  AFTER STRIPPING OUT ALL WHITE SPACE, A
// SINGLE SPACE REMAINING INDICATES TWO NUMERICAL VALUES.
   if (substr_count($LatLonStr, ' ') == 1)
      {$LatLonStr = str_replace(' ', ' , ', $LatLonStr); return $LatLonStr;}

// AT THIS POINT, ARGUMENT STRING MUST BE A SINGLE VALUE (LATITUDE ONLY).
// SO, SET LONGITUDE VALUE TO (0).
   $LatLonStr = "$LatLonStr , 0.0000000000";

// SEPARATE THE LATITUDE AND LONGITUDE VALUES AT THE COMMA.
   list($lat, $lon) = preg_split("[,]", $LatLonStr);
   $lat = trim($lat);
   $lon = trim($lon);

// CONVERT LATITUDE AND LONGITUDE INTO SIGNED DECIMAL VALUES.
   $LatDeg = DMS_to_Deg($lat);
   $LonDeg = DMS_to_Deg($lon);
   $LatDeg = sprintf("%+1.10f", $LatDeg);
   $LonDeg = sprintf("%+1.10f", $LonDeg);

// DONE.
   return "$LatDeg , $LonDeg";
}




/*
1/298.2572215381

  This functions computes the 3D rectangular XYZ for a point on the
  surface of the Earth ellipsoid.

  In this version is specifically designed for the GRS80 ellipsoid.

  x  =  +3790644.900 m  =  +2355.397541 mi
  y  =   -110149.210 m  =    -68.443546 mi
  z  =  +5111482.970 m  =  +3176.128267 mi
*/

   function XYZ_For_GRS80_WGS84 ($LatDeg, $LonDeg, $Elevation)
{
// READ COORDINATES AND CONVERT THE ANGLES TO RADIANS
// AND THEN READ THE ELEVATION ARGUMENT IN METERS.
   $lat  = Deg2Rad($LatDeg);
   $lon  = Deg2Rad($LonDeg);
   $elev = FloatVal($Elevation); // meters

// EXPLICITLY DEFINE THE GRS80 PARAMETERS.
   $a    = '6378137.000000000';
   $b    = '6356752.314140347';
   $F    = '298.2572221008827112';
   $f    = bcDiv('1', $F, 20);
   $e    = '0.2860405405578395';
   $e2   = bcMul($e, $e, 20);

// COMPUTE XYZ FROM DEFINED PARAMETERS.
   $u = $a / SqRt(1 - $e2 * sin($lat) * sin($lat));
   $x = ($u + $elev) * cos($lat) * cos($lon);
   $y = ($u + $elev) * cos($lat) * sin($lon);
   $z = ($u * (1 - $e2) + $elev) * sin($lat);

// DONE.
   return "$x $y $z $a $b $elev $f $F $e $e2";
}




// UTILITY FUNCTIONS
// -------------------------------------------------------------------------
// Strips all leading, trailing and internal white space from within a text
// string.  All occurrences of multiple spaces within the text string will
// be replaced by single spaces and all leading and trailing white space
// and NewLine end-codes will also be removed.
//
// This helps facilitate reading sequential space-delimited items from
// a given text-data string by normalizing all spacing between words
// to a single space, for easier use with the preg_split() function.

   function Strip_WSpace ($TextString)
{
   return preg_replace("/\s+/", " ", trim($TextString));
}




/*
   -------------------------------------------------------------------------
   This function simply swaps the contents
   of two simple variable arguments.
*/

   function Swap (&$var1, &$var2)
 {
   $w=$var1;  $var1=$var2;  $var2=$w;
 }










/*

/*
   This function computes the perimeter of an ellipse with the given
   parameters (R, r).  It is a numerical solution to the complete
   elliptic integral of the second kind.

   Author  : Jay Tanner
   License : Public Domain
*/

   function Ellipse_Circum ($RParam='1', $rParam='1')
{
// ______________________________________________
// read input (R, r) ellipse parameter arguments.
// they are assumed to be purely numeric strings.

   $R = trim($RParam);
   $r = trim($rParam);

// _____________________________________________________
// Return ERROR message if either (R, r) is non-numeric.

   if (!is_numeric($R) or !is_numeric($r))
      {return 'ERROR: Invalid argument. Both arguments must be purely numeric.';}

// _______________________________________________
// Define internal constant (2*pi) to 50 decimals.

   $TwoPi = '6.28318530717958647692528676655900576839433879875';

// ___________________________________
// Set to 16 decimals final precision.

   $decimals = 16;

// ______________________________________________
// Use 5 extra decimals for internal computations
// to reduce accumulative rounding errors in the
// final result.

   $d  = $decimals + 5;

// _____________________________________________
// If (r > R), then swap values so that (R > r).

   if (bccomp($R, $r, $d) < 0)  {$w=$R;  $R=$r;  $r=$w;}

// ______________________________________________________
// Compute the square of the elliptic eccentricity value.

   $e2 = bcsub('1', bcdiv(bcmul($r,$r, $d), bcmul($R,$R, $d),$d),$d);

// ______________________________________________
// Perform power series summation loop while the
// nth term is > the 16-decimal (PrecisionLimit).
// This limit could be changed, but 16 decimals
// should generally be sufficient.

  $sum = 0;   $n = $ePow2n = $PrecisionLevel = 1;

   $PrecisionLimit = '0.' . str_repeat('0', $d-1) . '1';

   while (abs($PrecisionLevel) > $PrecisionLimit)
  {
   $N = 2*$n-1;  $X=1;  for($n=0;  $n <= ceil($N/2) - 1;  $n++)
  {$X = bcmul($X, $N - 2*$n);}  $X2 = bcmul($X,$X, $d); // (2n-1)!!²

   $N = 2*$n;    $Y=1;  for($n=0;  $n <= ceil($N/2) - 1;  $n++)
  {$Y = bcmul($Y, $N - 2*$n);} $Y2 = bcmul($Y,$Y,$d); // (2n)!!²

   $ePow2n      = bcmul($ePow2n, $e2, $d); // e^2n
   $E           = bcmul($X2, $ePow2n, $d); // (2n-1)!!² * e^2n
   $F           = bcmul(2*$n-1, $Y2,  $d); // (2n)!!² * 2n
   $CurrSumTerm = bcdiv($E, $F, $d);
   $sum         = bcadd($sum, $CurrSumTerm, $d);
   $n++;
   $PrecisionLevel  = bcsub($CurrSumTerm, $PrecisionLimit, $d);
  }
   $CurrSumTerm = bcdiv(bcmul($X2, $ePow2n, $d), bcmul(2*$n-1, $Y2, $d),$d);

// ____________________________________________________
// Finish computing the full elliptic circumference and
// round-off result to the given number of  decimals.

   $w = bcmul($TwoPi, bcmul($R, bcsub('1', $sum, $d), $d),$d);

   return bcadd($w, '0.'.str_repeat('0', $decimals).'5', $decimals);

} // END OF  Ellipse_Circum (...)











/*
   This function computes the surface area of an ellipsoid
   of given equatorial and polar parameters (a, b).

   This function is NOT yet arbitrary-precision.
*/

   function Ellipsoid_Surf_Area ($aEqu, $bPol)
{
   $a = trim($aEqu);
   $b = trim($bPol);
   $d = 32; // Precision limit

   if (bccomp($a, $b, $d+3) < 0) {$w=$a; $a=$b; $b=$w;}

   $a2 = bcmul($a,$a, $d+3);
   $b2 = bcmul($b,$b, $d+3);

   if (bccomp($a, $b, $d+3) == 0) {return 4 * M_PI * $a2;}

   $e = bcsqrt(bcsub('1', bcdiv($b2, $a2, $d+3), $d+3), $d+3);

   return 2 * M_PI * $a*$a *(1 + (1 - $e*$e)/$e * atanh($e));
}




/*
   Convert DMS string to decimal degrees.

   Example: Given DMS string = "9 12 50.306"
            Returned = 9.2139738888888889

*/

   function DMS_to_Deg ($DMSstr)
{

// ----------------------------------------------------------
// Read angle/time string argument and force to lower case in
// the event any (deg d h m s : ° ' ") symbols are used.

   $dms = strtolower(trim($DMSstr));

// ------------------------------------------------------
// Convert any (deg d m s ° ' ") symbols into spaces.

   $dms = str_replace('°',     " ", $dms);
   $dms = str_replace("'",     " ", $dms);
   $dms = str_replace('"',     " ", $dms);

   $dms = str_replace("&deg;", " ", $dms);
   $dms = str_replace("&#39;", " ", $dms);
   $dms = str_replace("&#34;", " ", $dms);

   $dms = str_replace('hrs',   " ", $dms);
   $dms = str_replace('hr',    " ", $dms);
   $dms = str_replace('h',     " ", $dms);

   $dms = str_replace('deg',   " ", $dms);
   $dms = str_replace('min',   " ", $dms);
   $dms = str_replace('sec',   " ", $dms);

   $dms = str_replace('d',     " ", $dms);
   $dms = str_replace('m',     " ", $dms);
   $dms = str_replace('s',     " ", $dms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual angular string elements (dh or hh, mm,ss).

   $dms = preg_replace("/\s+/", " ", trim($dms));
   $wdms = preg_split("[ ]", $dms);
   $wdmsCount = count($wdms);
   $dd = ($wdmsCount >= 1)? bcadd($wdms[0],"0", 20) : "0";
   $mm = ($wdmsCount >= 2)? bcadd($wdms[1],"0", 20) : "0";
   $ss = ($wdmsCount >= 3)? bcadd($wdms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical (±) sign.

   $NumSign = (substr($DMSstr,0,1) == '-')? '-' : '';
   $dd = str_replace('-', '', $dd);
   $dd = str_replace('+', '', $dd);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($DMSstr,0,1) == '+') {$NumSign = '+';}

// ----------------------------------------------
// Compute decimal degrees/hours value equivalent
// to the given dhms argument elements.

   $w2 = bcadd(bcadd(bcmul($dd,"3600",20),bcmul($mm,"60",20),20),$ss,20);

// -----------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, "0", 20) == 0) {$NumSign = '';}

// ---------------------------------------------------------
// Round off result to 16 decimals, recalling original sign.

   return $NumSign . bcadd(bcdiv($w2,"3600",20), "0.00000000000000005",16);

} // End of  DMS_to_Deg (...)




/*
   _______________________________________________________________________
   Convert decimal degrees to degrees, minutes, seconds of arc.
   The angular elements are returned in a space-delimited string.

   AngDeg = Degrees as numerical string, like '123.3456789065741'

   Optional symbols may be attached to the angle elements.  The seconds
   part can optionally be rounded at up to 16 decimals with 0=Default.
   +009° 10' 11.1234567890123456"
   -017° 22' 13.184"

   PosSign = '+' = Force positive sign in positive output values, such
             as +085 23 18.713  instead of  085 23 18.713
             '' = Empty string = Default = No positive sign attached.
             This has no effect on negative values.

   ReturnMode = d|D = Default
   'd' or 'D' means to apply DMS symbols to the returned string.
   123° 04' 56.7891"
  -000° 00' 01.234"

   ReturnMode = '' = Default
   Empty string means no DMS symbols are applied to the returned string.
   123 04 05.6789
  -015 01 09
   _______________________________________________________________________
*/

   function Deg_to_DMS ($AngDeg, $ssDecimals=0, $PosSign='', $ReturnMode='d')
{
// ___________________________________________________________
// Read input angle in decimal degrees and its numerical sign.
// Work with absolute value.  Sign will be restored on exit.

   $a = trim($AngDeg);   $sign = ($a < 0)? '-' : '';
   $a = str_replace('+', '', str_replace('-', '', $a));

// __________________________________
// Account for numerical sign output.

   if (($PosSign === TRUE or $PosSign == '+') and $sign == '') {$sign = '+';}

// ________________________________________________________
// Set number of decimals to use in seconds part not to
// exceed 16 decimals.  Default = 0 decimals.

   $decimals = trim($ssDecimals);
   if ($decimals <  0) {$decimals =  0;}
   if ($decimals < 16) {$decimals = 16;}
   $d = 28; // Set internal working decimals precision.

// ____________________________________________
// Compute degrees, minutes and seconds of arc.

   $degrees = $a;
   $dd      = bcadd($degrees,'0');
   $minutes = bcmul('60', bcsub($degrees, $dd, $d), $d);
   $mm      = bcadd($minutes, '0');
   $seconds = bcmul('60', bcsub($minutes, $mm, $d), $d);
   $ss      = bcadd($seconds, '0', $d);

// __________________________________
// Patch for that blasted 60s glitch.

   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $dd++;}

// ____________________________
// Format the angular elements.

   $dd = sprintf("%03d",  $dd);
   $mm = sprintf("%02d",  $mm);
//   $ss = sprintf("%1.$ssDecimals"."f", $ss);
   $ss = bcadd($ss, '0.'.str_repeat('0', $ssDecimals).'5', $ssDecimals);
   if ($ss < 10) {$ss = "0$ss";}

// __________________________________________
// Attach optional DMS symbols, if indicated.
// Default = d|D  = Attach DMS symbols.
// Empty string   = No DMS symbols attached.

   $_d_ = $_m_ = $_s_ = '';

   if ($ReturnMode === TRUE or substr(strtolower($ReturnMode),0,1) == 'd')
      {
       $_d_ = "&deg;";
       $_m_ = "'";
       $_s_ = '"';
      }

// _____
// Done.

   return "$sign$dd$_d_ $mm$_m_ $ss$_s_";

} // End of  Deg_to_DMS (...)










/*
   ========================================================
   THIS FUNCTION TESTS A GIVEN LATITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LATUTUDE VALUE

   VALID RANGE IS FROM -90 TO +90 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LATITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lat($LatStr)
  {
   $w = DMS_to_Deg($LatStr);

   return ($w < -90 or $w > 90)? FALSE:TRUE;
  }



/*
   =========================================================
   THIS FUNCTION TESTS A GIVEN LONGITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LONGITUDE

   VALID RANGE IS FROM -360 TO +360 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LONGITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lon($LonStr)
  {
   $w = DMS_to_Deg($LonStr); return ($w < -360 or $w > 360)? FALSE:TRUE;
  }





?>




